% Check alignment after reprojection
%
% This script is part of
%
%    ISO/IEC JTC1/SC29/WG11 MPEG2018/M42415
%    April 2018, San Diego, CA, US

% Reset workspace
clear;
close all hidden;
framenr = 1; % Frame to check

% Open the metadata file
file = fopen('ClassroomVideo.json');
metadata = jsondecode(char(fread(file, inf)'));
fclose(file);

nviews = length(metadata.cameras);
normdisp = cell(nviews, 1);
raylength = cell(nviews, 1);
coordinate_xyz = cell(nviews, 1);

for viewnr = 1:nviews
    C = metadata.cameras(viewnr);

    % Load the depth maps (encoded as normalized disparity)
    filepath = sprintf('output/raw/%s_4096_2048_0_8_1000_0_%04d.depth', C.Name, framenr);
    file = fopen(filepath);
    normdisp{viewnr} = fread(file, C.Resolution', 'uint16', 'l')';
    fclose(file);

    % Convert to depth (ray length) in meters
    raylength{viewnr} = C.Rmin * (2^16 - 1) ./ normdisp{viewnr};
    
    % Equirectangular projection
    hPos = 0.5 : C.Resolution(1);
    vPos = 0.5 : C.Resolution(2);
    [hPos, vPos] = meshgrid(hPos, vPos);
    phi = 2*pi*(0.5 - hPos / C.Resolution(1)); % (5-1)
    theta = pi*(0.5 - vPos / C.Resolution(2));
    
    % Ray direction
    raydirection_xyz = cat(3, ...
        cos(phi) .* cos(theta), ...
        sin(phi) .* cos(theta), ...
        sin(theta));
    
    % Ray origin
    rayorigin_xyz = reshape(C.Position, [1 1 3]);
    
    % Coordinate
    coordinate_xyz{viewnr} = bsxfun(@plus, rayorigin_xyz, ...
        bsxfun(@times, raylength{viewnr}, raydirection_xyz));
end

% Create a point cloud from middle scanlines
cloud_xyz = cell(nviews, 1);
for viewnr = 1:nviews
    mask = raylength{viewnr}(end/2,:) < 9;
    cloud_xyz{viewnr} = permute(coordinate_xyz{viewnr}(end/2,mask,:), [2 3 1]);
end
cloud_xyz = cat(1, cloud_xyz{:});

% Quantative alignment test
roi_x = [-4.1 -3.9];
roi_y = [-2.7  3.2];
selection = roi_x(1) < cloud_xyz(:,1) & cloud_xyz(:,1) < roi_x(2) ...
          & roi_y(1) < cloud_xyz(:,2) & cloud_xyz(:,2) < roi_y(2);
X = cloud_xyz(selection,1);
rmse = sqrt(mean((X + 4).^2));
fprintf('Wall: mean=%g m, std=%g um\n', mean(X), 1e6*std(X));
% Wall: mean=-3.99581 m, std=104.279 um

% Visual alignment check
fig = figure(...
    'Name', 'AlignmentCheck', ...
    'WindowStyle', 'docked', ...
    'PaperUnits', 'centimeters', ...
    'PaperPosition', [0 0 21 15]);
scatter3(cloud_xyz(:,1), cloud_xyz(:,2), cloud_xyz(:,3), '.'); 
daspect([1 1 1]); 
xlabel('x [m] (front-to-back)'); 
ylabel('y [m] (lateral, side-to-side)');
zlabel('z [m] (vertical, up)');
hold on;
plot(roi_x([1 2 2 1 1]), roi_y([1 1 2 2 1]), 'r');
t = linspace(-pi, pi, 100);
plot(metadata.Informative.BoundingBox_size/2*cos(t), metadata.Informative.BoundingBox_size/2*sin(t), 'k');
hold off;
grid;
view(-90, 90); % Appearance of a 2-D plot
legend('Reprojected points', 'Alignment check', 'Bounding region', 'Location', 'NorthWest');
saveas(fig, 'AlignmentCheck.png');